@@ -14,11 +14,17 @@ extern int cm_getLocalMountPointChange();
 extern int cm_reInitLocalMountPoints();
 extern void cm_InitFreelance();
 extern void cm_FreelanceShutdown(void);
-extern int cm_noteLocalMountPointChange(void);
+extern int cm_noteLocalMountPointChange(afs_int32 locked);
 extern long cm_FreelanceRemoveMount(char *toremove);
 extern long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw, cm_fid_t *fidp);
 extern long cm_FreelanceRemoveSymlink(char *toremove);
 extern long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp);
+extern long cm_FreelanceMountPointExists(char * filename, int prefix_ok);
+extern long cm_FreelanceSymlinkExists(char * filename, int prefix_ok);
+extern long cm_FreelanceFetchMountPointString(cm_scache_t *scp);
+extern long cm_FreelanceFetchFileType(cm_scache_t *scp);
+extern void cm_FreelanceImportCellServDB(void);
+
 extern int cm_clearLocalMountPointChange();
 extern int cm_FakeRootFid(cm_fid_t *fidp);
 
@@ -27,4 +33,6 @@ extern int cm_FakeRootFid(cm_fid_t *fidp);
 #define AFS_FAKE_ROOT_VOL_ID  0xFFFFFFFF
 
 extern time_t FakeFreelanceModTime;
+extern int cm_freelanceEnabled;
+extern int cm_freelanceImportCellServDB;
 #endif // _CM_FREELANCE_H
