@@ -1,14 +1,14 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef __SMB_IOCONS_H_ENV_
-#define __SMB_IOCONS_H_ENV_ 1
+#ifndef OPENAFS_WINNT_AFSD_SMB_IOCONS_H
+#define OPENAFS_WINNT_AFSD_SMB_IOCONS_H 1
 
 /* included in both AFSD and fs commands */
 
@@ -95,15 +95,29 @@ struct sbstruct {
 #define VIOC_PATH_AVAILABILITY          0x31
 #define VIOC_GETFILETYPE                0x32
 #define VIOC_UNICODECTL                 0x33
+#define VIOC_SETOWNER                   0x34
+#define VIOC_SETGROUP                   0x35
+#define VIOC_FS_CMD                     0x36
+#define VIOCNEWCELL2                    0x37
+#define VIOC_GETUNIXMODE                0x38
+#define VIOC_SETUNIXMODE                0x39
+#define VIOC_GETVERIFYDATA              0x3A
+#define VIOC_SETVERIFYDATA              0x3B
 
 #define VIOC_VOLSTAT_TEST               0x3F
 
+#define VIOC_NEWCELL2_FLAG_LINKED       0x1
+#define VIOC_NEWCELL2_FLAG_USEDNS       0x2
+#define VIOC_NEWCELL2_FLAG_USEREG       0x4
+
 /* magic file name for ioctl opens */
 #define CM_IOCTL_FILENAME	"\\_._AFS_IOCTL_._"	/* double backslashes for C compiler */
+#define CM_IOCTL_FILENAME_W	L"\\_._AFS_IOCTL_._"	/* double backslashes for C compiler */
 #define CM_IOCTL_FILENAME_NOSLASH "_._AFS_IOCTL_._"
+#define CM_IOCTL_FILENAME_NOSLASH_W L"_._AFS_IOCTL_._"
 
 /* max parms for ioctl, in either direction */
 #define CM_IOCTL_MAXDATA	        8192*2
 #define CM_IOCTL_MAXPROCS               64
 
-#endif /*  __SMB_IOCONS_H_ENV_ */
+#endif /*  OPENAFS_WINNT_AFSD_SMB_IOCONS_H */
