@@ -89,12 +89,14 @@ AFSDirControl( IN PDEVICE_OBJECT LibDeviceObject,
                 break;
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
     {
 
         AFSDbgLogMsg( 0,
                       0,
                       "EXCEPTION - AFSDirControl\n");
+
+        AFSDumpTraceFilesFnc();
     }
 
     if( ntStatus != STATUS_PENDING)
@@ -724,7 +726,6 @@ AFSQueryDirectory( IN PIRP Irp)
 
             AFSValidateEntry( pDirEntry,
                               &pCcb->AuthGroup,
-                              FALSE,
                               FALSE);
 
             pObjectInfo = pDirEntry->ObjectInformation;
