@@ -17,8 +17,8 @@
 
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
-#include "h/mm.h"
-#include "h/slab.h"
+#include <linux/mm.h>
+#include <linux/slab.h>
 
 #include "afs_atomlist.h"
 #include "afs_lhash.h"
@@ -49,14 +49,7 @@ unsigned int afs_linux_hash_verify_count = 0;	/* used by hash_verify */
 struct afs_lhash_stat afs_linux_lsb;	/* hash table statistics */
 unsigned int afs_linux_hash_bucket_dist[MAX_BUCKET_LEN];	/* bucket population distribution in our hash table */
 
-#if defined(AFS_LINUX24_ENV)
-#include "h/vmalloc.h"
-#else
-/* externs : can we do this in a better way. Including vmalloc.h causes other
- * problems.*/
-extern void vfree(void *addr);
-extern void *vmalloc(unsigned long size);
-#endif
+#include <linux/vmalloc.h>
 
 /* Allocator support functions (static) */
 
@@ -86,13 +79,7 @@ linux_alloc(unsigned int asize, int drop_glock)
     /*  if we can use kmalloc use it to allocate the required memory. */
     while (!new && max_retry) {
 	if (asize <= MAX_KMALLOC_SIZE) {
-	    new = (void *)(unsigned long)kmalloc(asize,
-#ifdef GFP_NOFS
-						 GFP_NOFS
-#else
-						 GFP_KERNEL
-#endif
-		);
+	    new = (void *)(unsigned long)kmalloc(asize, GFP_NOFS);
 	    if (new)		/* piggy back alloc type */
 		new = (void *)(KM_TYPE | (unsigned long)new);
 	} else {
@@ -284,10 +271,10 @@ get_hash_stats(void)
 
 /************** Linux memory allocator interface functions **********/
 
-#if defined(AFS_LINUX24_ENV)
-DECLARE_MUTEX(afs_linux_alloc_sem);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+DEFINE_MUTEX(afs_linux_alloc_sem);
 #else
-struct semaphore afs_linux_alloc_sem = MUTEX;
+DECLARE_MUTEX(afs_linux_alloc_sem);
 #endif
 
 void *
@@ -303,7 +290,7 @@ osi_linux_alloc(unsigned int asize, int drop_glock)
 	return new;
     }
 
-    down(&afs_linux_alloc_sem);
+    mutex_lock(&afs_linux_alloc_sem);
 
     /* allocator hasn't been initialized yet */
     if (allocator_init == 0) {
@@ -333,7 +320,7 @@ osi_linux_alloc(unsigned int asize, int drop_glock)
 	get_hash_stats();
     }
   error:
-    up(&afs_linux_alloc_sem);
+    mutex_unlock(&afs_linux_alloc_sem);
     return MEMADDR(new);
 
   free_error:
@@ -353,7 +340,7 @@ osi_linux_free(void *addr)
 {
     struct osi_linux_mem lmem, *lmp;
 
-    down(&afs_linux_alloc_sem);
+    mutex_lock(&afs_linux_alloc_sem);
 
     lmem.chunk = addr;
     /* remove this chunk from our hash table */
@@ -364,11 +351,11 @@ osi_linux_free(void *addr)
 	afs_atomlist_put(al_mem_pool, lmp);	/* return osi_linux_mem struct to pool */
 	afs_linux_cur_allocs--;
     } else {
-	BUG();
 	printf("osi_linux_free: failed to remove chunk from hashtable\n");
+	BUG();
     }
 
-    up(&afs_linux_alloc_sem);
+    mutex_unlock(&afs_linux_alloc_sem);
 }
 
 /* osi_linux_free_afs_memory() - free all chunks of memory allocated.
@@ -376,7 +363,7 @@ osi_linux_free(void *addr)
 void
 osi_linux_free_afs_memory(void)
 {
-    down(&afs_linux_alloc_sem);
+    mutex_lock(&afs_linux_alloc_sem);
 
     if (allocator_init) {
 	/* iterate through all elements in the hash table and free both 
@@ -393,7 +380,7 @@ osi_linux_free_afs_memory(void)
 	/* change the state so that the allocator is now uninitialized. */
 	allocator_init = 0;
     }
-    up(&afs_linux_alloc_sem);
+    mutex_unlock(&afs_linux_alloc_sem);
 }
 
 /* osi_linux_verify_alloced_memory(): verify all chunks of alloced memory in
@@ -402,7 +389,7 @@ osi_linux_free_afs_memory(void)
 void
 osi_linux_verify_alloced_memory()
 {
-    down(&afs_linux_alloc_sem);
+    mutex_lock(&afs_linux_alloc_sem);
 
     /* count of times hash_verify was called. reset it to 0 before iteration */
     afs_linux_hash_verify_count = 0;
@@ -417,6 +404,37 @@ osi_linux_verify_alloced_memory()
 	     afs_linux_hash_verify_count - afs_linux_cur_allocs);
     }
 
-    up(&afs_linux_alloc_sem);
+    mutex_unlock(&afs_linux_alloc_sem);
     return;
 }
+
+#ifdef AFS_PRIVATE_OSI_ALLOCSPACES
+
+void
+osi_FreeLargeSpace(void *p)
+{
+    kfree(p);
+}
+
+void
+osi_FreeSmallSpace(void *p)
+{
+    kfree(p);
+}
+
+void *
+osi_AllocLargeSpace(size_t size)
+{
+    if (size > AFS_LRALLOCSIZ)
+	osi_Panic("osi_AllocLargeSpace: size=%d\n", (int) size);
+    return kmalloc(AFS_LRALLOCSIZ, GFP_NOFS);
+}
+
+void *
+osi_AllocSmallSpace(size_t size)
+{
+    if (size > AFS_SMALLOCSIZ)
+	osi_Panic("osi_AllocSmallS: size=%d\n", (int)size);
+    return kmalloc(AFS_SMALLOCSIZ, GFP_NOFS);
+}
+#endif /* AFS_PRIVATE_OSI_ALLOCSPACES */
