@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,32 +10,18 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/bozo/fsbnodeops.c,v 1.15.2.2 2007/10/31 04:21:14 shadow Exp $");
+#include <afs/procmgmt.h>
+#include <roken.h>
+#include <afs/opr.h>
 
-#include <sys/types.h>
 #include <lwp.h>
-#include <errno.h>
-#include <stdio.h>
-#ifdef	AFS_SUN5_ENV
-#include <fcntl.h>
-#endif
-#ifdef AFS_NT40_ENV
-#include <io.h>
-#include <fcntl.h>
-#else
-#include <sys/file.h>
-
-#include <string.h>
-#include <stdlib.h>
-
-#endif /* AFS_NT40_ENV */
-#include <sys/stat.h>
-#include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
+#include <rx/rx.h>
 #include <afs/afsutil.h>
+
 #include "bnode.h"
 #include "bosprototypes.h"
 
+extern char *DoPidFiles;
 static int emergency = 0;
 
 /* if this file exists, then we have to salvage the file system */
@@ -46,20 +32,20 @@ static int emergency = 0;
 
 /*  basic rules:
     Normal operation involves having the file server and the vol server both running.
-    
+
     If the vol server terminates, it can simply be restarted.
-    
+
     If the file server terminates, the disk must salvaged before the file server
     can be restarted.  In order to restart either the file server or the salvager,
     the vol server must be shut down.
-    
+
     If the file server terminates *normally* (exits after receiving a SIGQUIT)
     then we don't have to salvage it.
-    
+
     The needsSalvage flag is set when the file server is started.  It is cleared
     if the file server exits when fileSDW is true but fileKillSent is false,
     indicating that it exited after receiving a quit, but before we sent it a kill.
-    
+
     The needsSalvage flag is cleared when the salvager exits.
 */
 
@@ -99,41 +85,36 @@ struct fsbnode {
     char needsClock;		/* do we need clock ticks */
 };
 
-
-
-struct bnode * fs_create(char *ainstance, char *afilecmd, char *avolcmd, 
-			 char *asalcmd, char *ascancmd);
-struct bnode * dafs_create(char *ainstance, char *afilecmd, char *avolcmd, 
+struct bnode * fs_create(char *ainstance, char *afilecmd, char *avolcmd,
+			 char *asalcmd, char *ascancmd, char *dummy);
+struct bnode * dafs_create(char *ainstance, char *afilecmd, char *avolcmd,
 			   char * asalsrvcmd, char *asalcmd, char *ascancmd);
 
-static int fs_hascore(register struct ezbnode *abnode);
-static int fs_restartp(register struct fsbnode *abnode);
-static int SetSalFlag(register struct fsbnode *abnode, register int aflag);
-static int RestoreSalFlag(register struct fsbnode *abnode);
-static int fs_delete(struct fsbnode *abnode);
-static int fs_timeout(struct fsbnode *abnode);
-static int fs_getstat(struct fsbnode *abnode, afs_int32 * astatus);
-static int fs_setstat(register struct fsbnode *abnode, afs_int32 astatus);
-static int fs_procexit(struct fsbnode *abnode, struct bnode_proc *aproc);
-static int fs_getstring(struct fsbnode *abnode, char *abuffer, afs_int32 alen);
-
-
-static int fs_getparm(struct fsbnode *abnode, afs_int32 aindex, 
+static int fs_hascore(struct bnode *abnode);
+static int fs_restartp(struct bnode *abnode);
+static int fs_delete(struct bnode *abnode);
+static int fs_timeout(struct bnode *abnode);
+static int fs_getstat(struct bnode *abnode, afs_int32 * astatus);
+static int fs_setstat(struct bnode *abnode, afs_int32 astatus);
+static int fs_procstarted(struct bnode *abnode, struct bnode_proc *aproc);
+static int fs_procexit(struct bnode *abnode, struct bnode_proc *aproc);
+static int fs_getstring(struct bnode *abnode, char *abuffer, afs_int32 alen);
+static int fs_getparm(struct bnode *abnode, afs_int32 aindex,
 		      char *abuffer, afs_int32 alen);
-static int dafs_getparm(struct fsbnode *abnode, afs_int32 aindex, 
+static int dafs_getparm(struct bnode *abnode, afs_int32 aindex,
 			char *abuffer, afs_int32 alen);
 
+static int SetSalFlag(struct fsbnode *abnode, int aflag);
+static int RestoreSalFlag(struct fsbnode *abnode);
+static void SetNeedsClock(struct fsbnode *);
+static int NudgeProcs(struct fsbnode *);
+
 #ifdef AFS_NT40_ENV
 static void AppendExecutableExtension(char *cmd);
 #else
 #define AppendExecutableExtension(x)
 #endif
 
-static void SetNeedsClock(register struct fsbnode *ab);
-static int NudgeProcs(register struct fsbnode *abnode);
-
-
-
 struct bnode_ops fsbnode_ops = {
     fs_create,
     fs_timeout,
@@ -145,6 +126,7 @@ struct bnode_ops fsbnode_ops = {
     fs_getparm,
     fs_restartp,
     fs_hascore,
+    fs_procstarted,
 };
 
 /* demand attach fs bnode ops */
@@ -159,8 +141,17 @@ struct bnode_ops dafsbnode_ops = {
     dafs_getparm,
     fs_restartp,
     fs_hascore,
+    fs_procstarted,
 };
 
+/* Quick inline function to safely convert a fsbnode to a bnode without
+ * dropping type information
+ */
+
+static_inline struct bnode *
+fsbnode2bnode(struct fsbnode *abnode) {
+    return (struct bnode *) abnode;
+}
 
 /* Function to tell whether this bnode has a core file or not.  You might
  * think that this could be in bnode.c, and decide what core files to check
@@ -169,7 +160,7 @@ struct bnode_ops dafsbnode_ops = {
  * time the query is done.
  */
 static int
-fs_hascore(register struct ezbnode *abnode)
+fs_hascore(struct bnode *abnode)
 {
     char tbuffer[256];
 
@@ -203,10 +194,11 @@ fs_hascore(register struct ezbnode *abnode)
 }
 
 static int
-fs_restartp(register struct fsbnode *abnode)
+fs_restartp(struct bnode *bn)
 {
+    struct fsbnode *abnode = (struct fsbnode *)bn;
     struct bnode_token *tt;
-    register afs_int32 code;
+    afs_int32 code;
     struct stat tstat;
 
     code = bnode_ParseLine(abnode->filecmd, &tt);
@@ -258,7 +250,7 @@ fs_restartp(register struct fsbnode *abnode)
 	    bnode_FreeTokens(tt);
 	    return 0;
 	}
-	if (tstat.st_ctime > abnode->lastScanStart)
+	if (tstat.st_ctime > abnode->lastSalsrvStart)
 	    code = 1;
 	else
 	    code = 0;
@@ -290,7 +282,7 @@ fs_restartp(register struct fsbnode *abnode)
 /* set needsSalvage flag, creating file SALVAGE.<instancename> if
     we need to salvage the file system (so we can tell over panic reboots */
 static int
-SetSalFlag(register struct fsbnode *abnode, register int aflag)
+SetSalFlag(struct fsbnode *abnode, int aflag)
 {
     char tbuffer[AFSDIR_PATH_MAX];
     int fd;
@@ -312,7 +304,7 @@ SetSalFlag(register struct fsbnode *abnode, register int aflag)
 
 /* set the needsSalvage flag according to the existence of the salvage file */
 static int
-RestoreSalFlag(register struct fsbnode *abnode)
+RestoreSalFlag(struct fsbnode *abnode)
 {
     char tbuffer[AFSDIR_PATH_MAX];
 
@@ -333,17 +325,19 @@ RestoreSalFlag(register struct fsbnode *abnode)
 }
 
 char *
-copystr(register char *a)
+copystr(char *a)
 {
-    register char *b;
+    char *b;
     b = (char *)malloc(strlen(a) + 1);
     strcpy(b, a);
     return b;
 }
 
 static int
-fs_delete(struct fsbnode *abnode)
+fs_delete(struct bnode *bn)
 {
+    struct fsbnode *abnode = (struct fsbnode *)bn;
+
     free(abnode->filecmd);
     free(abnode->volcmd);
     free(abnode->salcmd);
@@ -373,15 +367,16 @@ AppendExecutableExtension(char *cmd)
 
 struct bnode *
 fs_create(char *ainstance, char *afilecmd, char *avolcmd, char *asalcmd,
-	  char *ascancmd)
+	  char *ascancmd, char *dummy)
 {
     struct stat tstat;
-    register struct fsbnode *te;
+    struct fsbnode *te;
     char cmdname[AFSDIR_PATH_MAX];
     char *fileCmdpath, *volCmdpath, *salCmdpath, *scanCmdpath;
     int bailout = 0;
 
-    te = fileCmdpath = volCmdpath = salCmdpath = scanCmdpath = NULL;
+    fileCmdpath = volCmdpath = salCmdpath = scanCmdpath = NULL;
+    te = NULL;
 
     /* construct local paths from canonical (wire-format) paths */
     if (ConstructLocalBinPath(afilecmd, &fileCmdpath)) {
@@ -458,11 +453,12 @@ fs_create(char *ainstance, char *afilecmd, char *avolcmd, char *asalcmd,
 	te->scancmd = scanCmdpath;
     else
 	te->scancmd = NULL;
-    if (bnode_InitBnode(te, &fsbnode_ops, ainstance) != 0) {
+    if (bnode_InitBnode(fsbnode2bnode(te), &fsbnode_ops, ainstance) != 0) {
 	bailout = 1;
 	goto done;
     }
-    bnode_SetTimeout(te, POLLTIME);	/* ask for timeout activations every 10 seconds */
+    bnode_SetTimeout(fsbnode2bnode(te), POLLTIME);
+    		/* ask for timeout activations every 10 seconds */
     RestoreSalFlag(te);		/* restore needsSalvage flag based on file's existence */
     SetNeedsClock(te);		/* compute needsClock field */
 
@@ -481,21 +477,22 @@ fs_create(char *ainstance, char *afilecmd, char *avolcmd, char *asalcmd,
 	return NULL;
     }
 
-    return (struct bnode *)te;
+    return fsbnode2bnode(te);
 }
 
 /* create a demand attach fs bnode */
 struct bnode *
-dafs_create(char *ainstance, char *afilecmd, char *avolcmd, 
+dafs_create(char *ainstance, char *afilecmd, char *avolcmd,
 	    char * asalsrvcmd, char *asalcmd, char *ascancmd)
 {
     struct stat tstat;
-    register struct fsbnode *te;
+    struct fsbnode *te;
     char cmdname[AFSDIR_PATH_MAX];
     char *fileCmdpath, *volCmdpath, *salsrvCmdpath, *salCmdpath, *scanCmdpath;
     int bailout = 0;
 
-    te = fileCmdpath = volCmdpath = salsrvCmdpath = salCmdpath = scanCmdpath = NULL;
+    fileCmdpath = volCmdpath = salsrvCmdpath = salCmdpath = scanCmdpath = NULL;
+    te = NULL;
 
     /* construct local paths from canonical (wire-format) paths */
     if (ConstructLocalBinPath(afilecmd, &fileCmdpath)) {
@@ -585,11 +582,12 @@ dafs_create(char *ainstance, char *afilecmd, char *avolcmd,
 	te->scancmd = scanCmdpath;
     else
 	te->scancmd = NULL;
-    if (bnode_InitBnode(te, &dafsbnode_ops, ainstance) != 0) {
+    if (bnode_InitBnode(fsbnode2bnode(te), &dafsbnode_ops, ainstance) != 0) {
 	bailout = 1;
 	goto done;
     }
-    bnode_SetTimeout(te, POLLTIME);	/* ask for timeout activations every 10 seconds */
+    bnode_SetTimeout(fsbnode2bnode(te), POLLTIME);
+    		/* ask for timeout activations every 10 seconds */
     RestoreSalFlag(te);		/* restore needsSalvage flag based on file's existence */
     SetNeedsClock(te);		/* compute needsClock field */
 
@@ -610,14 +608,16 @@ dafs_create(char *ainstance, char *afilecmd, char *avolcmd,
 	return NULL;
     }
 
-    return (struct bnode *)te;
+    return fsbnode2bnode(te);
 }
 
 /* called to SIGKILL a process if it doesn't terminate normally */
 static int
-fs_timeout(struct fsbnode *abnode)
+fs_timeout(struct bnode *bn)
 {
-    register afs_int32 now;
+    struct fsbnode *abnode = (struct fsbnode *)bn;
+
+    afs_int32 now;
 
     now = FT_ApproxTime();
     /* shutting down */
@@ -671,9 +671,11 @@ fs_timeout(struct fsbnode *abnode)
 }
 
 static int
-fs_getstat(struct fsbnode *abnode, afs_int32 * astatus)
+fs_getstat(struct bnode *bn, afs_int32 * astatus)
 {
-    register afs_int32 temp;
+    struct fsbnode *abnode = (struct fsbnode *) bn;
+
+    afs_int32 temp;
     if (abnode->volSDW || abnode->fileSDW || abnode->salSDW
 	|| abnode->scanSDW || abnode->salsrvSDW)
 	temp = BSTAT_SHUTTINGDOWN;
@@ -694,16 +696,33 @@ fs_getstat(struct fsbnode *abnode, afs_int32 * astatus)
 }
 
 static int
-fs_setstat(register struct fsbnode *abnode, afs_int32 astatus)
+fs_setstat(struct bnode *abnode, afs_int32 astatus)
 {
-    return NudgeProcs(abnode);
+    return NudgeProcs((struct fsbnode *) abnode);
 }
 
 static int
-fs_procexit(struct fsbnode *abnode, struct bnode_proc *aproc)
+fs_procstarted(struct bnode *bn, struct bnode_proc *aproc)
 {
+    int code = 0;
+
+    if (DoPidFiles) {
+	code = bozo_CreatePidFile(bn->name, aproc->coreName, aproc->pid);
+    }
+    return code;
+}
+
+static int
+fs_procexit(struct bnode *bn, struct bnode_proc *aproc)
+{
+   struct fsbnode *abnode = (struct fsbnode *)bn;
+
     /* process has exited */
 
+    if (DoPidFiles) {
+	bozo_DeletePidFile(bn->name, aproc->coreName);
+    }
+
     if (aproc == abnode->volProc) {
 	abnode->volProc = 0;
 	abnode->volRunning = 0;
@@ -749,7 +768,7 @@ fs_procexit(struct fsbnode *abnode, struct bnode_proc *aproc)
 
 /* make sure we're periodically checking the state if we need to */
 static void
-SetNeedsClock(register struct fsbnode *ab)
+SetNeedsClock(struct fsbnode *ab)
 {
     if (ab->b.goal == 1 && ab->fileRunning && ab->volRunning
 	&& (!ab->scancmd || ab->scanRunning)
@@ -760,17 +779,17 @@ SetNeedsClock(register struct fsbnode *ab)
 	ab->needsClock = 0;	/* halted normally */
     else
 	ab->needsClock = 1;	/* other */
-    if (ab->needsClock && !bnode_PendingTimeout(ab))
-	bnode_SetTimeout(ab, POLLTIME);
+    if (ab->needsClock && !bnode_PendingTimeout(fsbnode2bnode(ab)))
+	bnode_SetTimeout(fsbnode2bnode(ab), POLLTIME);
     if (!ab->needsClock)
-	bnode_SetTimeout(ab, 0);
+	bnode_SetTimeout(fsbnode2bnode(ab), 0);
 }
 
 static int
-NudgeProcs(register struct fsbnode *abnode)
+NudgeProcs(struct fsbnode *abnode)
 {
     struct bnode_proc *tp;	/* not register */
-    register afs_int32 code;
+    afs_int32 code;
     afs_int32 now;
 
     now = FT_ApproxTime();
@@ -783,14 +802,14 @@ NudgeProcs(register struct fsbnode *abnode)
 		bozo_Log("Salvager running along with file server!\n");
 		bozo_Log("Emergency shutdown\n");
 		emergency = 1;
-		bnode_SetGoal(abnode, BSTAT_SHUTDOWN);
+		bnode_SetGoal(fsbnode2bnode(abnode), BSTAT_SHUTDOWN);
 		bnode_StopProc(abnode->salProc, SIGKILL);
 		SetNeedsClock(abnode);
 		return -1;
 	    }
 	    if (!abnode->volRunning) {
 		abnode->lastVolStart = FT_ApproxTime();
-		code = bnode_NewProc(abnode, abnode->volcmd, "vol", &tp);
+		code = bnode_NewProc(fsbnode2bnode(abnode), abnode->volcmd, "vol", &tp);
 		if (code == 0) {
 		    abnode->volProc = tp;
 		    abnode->volRunning = 1;
@@ -800,7 +819,7 @@ NudgeProcs(register struct fsbnode *abnode)
 		if (!abnode->salsrvRunning) {
 		    abnode->lastSalsrvStart = FT_ApproxTime();
 		    code =
-			bnode_NewProc(abnode, abnode->salsrvcmd, "salsrv",
+			bnode_NewProc(fsbnode2bnode(abnode), abnode->salsrvcmd, "salsrv",
 				      &tp);
 		    if (code == 0) {
 			abnode->salsrvProc = tp;
@@ -812,7 +831,7 @@ NudgeProcs(register struct fsbnode *abnode)
 		if (!abnode->scanRunning) {
 		    abnode->lastScanStart = FT_ApproxTime();
 		    code =
-			bnode_NewProc(abnode, abnode->scancmd, "scanner",
+			bnode_NewProc(fsbnode2bnode(abnode), abnode->scancmd, "scanner",
 				      &tp);
 		    if (code == 0) {
 			abnode->scanProc = tp;
@@ -828,7 +847,7 @@ NudgeProcs(register struct fsbnode *abnode)
 		if (!abnode->fileRunning) {
 		    abnode->lastFileStart = FT_ApproxTime();
 		    code =
-			bnode_NewProc(abnode, abnode->filecmd, "file", &tp);
+			bnode_NewProc(fsbnode2bnode(abnode), abnode->filecmd, "file", &tp);
 		    if (code == 0) {
 			abnode->fileProc = tp;
 			abnode->fileRunning = 1;
@@ -837,7 +856,7 @@ NudgeProcs(register struct fsbnode *abnode)
 		}
 		if (!abnode->volRunning) {
 		    abnode->lastVolStart = FT_ApproxTime();
-		    code = bnode_NewProc(abnode, abnode->volcmd, "vol", &tp);
+		    code = bnode_NewProc(fsbnode2bnode(abnode), abnode->volcmd, "vol", &tp);
 		    if (code == 0) {
 			abnode->volProc = tp;
 			abnode->volRunning = 1;
@@ -846,7 +865,7 @@ NudgeProcs(register struct fsbnode *abnode)
 		if (abnode->salsrvcmd && !abnode->salsrvRunning) {
 		    abnode->lastSalsrvStart = FT_ApproxTime();
 		    code =
-			bnode_NewProc(abnode, abnode->salsrvcmd, "salsrv",
+			bnode_NewProc(fsbnode2bnode(abnode), abnode->salsrvcmd, "salsrv",
 				      &tp);
 		    if (code == 0) {
 			abnode->salsrvProc = tp;
@@ -856,7 +875,7 @@ NudgeProcs(register struct fsbnode *abnode)
 		if (abnode->scancmd && !abnode->scanRunning) {
 		    abnode->lastScanStart = FT_ApproxTime();
 		    code =
-			bnode_NewProc(abnode, abnode->scancmd, "scanner",
+			bnode_NewProc(fsbnode2bnode(abnode), abnode->scancmd, "scanner",
 				      &tp);
 		    if (code == 0) {
 			abnode->scanProc = tp;
@@ -888,7 +907,7 @@ NudgeProcs(register struct fsbnode *abnode)
 		    return 0;
 		/* otherwise, it is safe to start salvager */
 		if (!abnode->salRunning) {
-		    code = bnode_NewProc(abnode, abnode->salcmd, "salv", &tp);
+		    code = bnode_NewProc(fsbnode2bnode(abnode), abnode->salcmd, "salv", &tp);
 		    if (code == 0) {
 			abnode->salProc = tp;
 			abnode->salRunning = 1;
@@ -929,8 +948,10 @@ NudgeProcs(register struct fsbnode *abnode)
 }
 
 static int
-fs_getstring(struct fsbnode *abnode, char *abuffer, afs_int32 alen)
+fs_getstring(struct bnode *bn, char *abuffer, afs_int32 alen)
 {
+    struct fsbnode *abnode = (struct fsbnode *)bn;
+
     if (alen < 40)
 	return -1;
     if (abnode->b.goal == 1) {
@@ -971,9 +992,11 @@ fs_getstring(struct fsbnode *abnode, char *abuffer, afs_int32 alen)
 }
 
 static int
-fs_getparm(struct fsbnode *abnode, afs_int32 aindex, char *abuffer,
+fs_getparm(struct bnode *bn, afs_int32 aindex, char *abuffer,
 	   afs_int32 alen)
 {
+    struct fsbnode *abnode = (struct fsbnode *)bn;
+
     if (aindex == 0)
 	strcpy(abuffer, abnode->filecmd);
     else if (aindex == 1)
@@ -988,9 +1011,11 @@ fs_getparm(struct fsbnode *abnode, afs_int32 aindex, char *abuffer,
 }
 
 static int
-dafs_getparm(struct fsbnode *abnode, afs_int32 aindex, char *abuffer,
+dafs_getparm(struct bnode *bn, afs_int32 aindex, char *abuffer,
 	     afs_int32 alen)
 {
+    struct fsbnode *abnode = (struct fsbnode *)bn;
+
     if (aindex == 0)
 	strcpy(abuffer, abnode->filecmd);
     else if (aindex == 1)
