@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -13,25 +13,17 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/kauth/knfs.c,v 1.8.14.2 2007/10/31 04:09:30 shadow Exp $");
-
-#include <stdio.h>
 #include <afs/stds.h>
-#include <sys/types.h>
+
+#include <roken.h>
+
 #include <ctype.h>
-#include <sys/stat.h>
-#include <signal.h>
-#include <time.h>
-#include <netdb.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <string.h>
+
 #include <afs/vice.h>
 #include <afs/cmd.h>
 #include <afs/auth.h>
 #include <afs/afsutil.h>
+#include <afs/sys_prototypes.h>
 
 /*
 Modifications:
@@ -52,11 +44,8 @@ struct ClearToken {
 };
 
 
-static
-SetSysname(ahost, auid, sysname)
-     afs_int32 ahost;
-     afs_int32 auid;
-     char *sysname;
+static int
+SetSysname(afs_int32 ahost, afs_int32 auid, char *sysname)
 {
     afs_int32 code;
     afs_int32 pheader[6];
@@ -95,19 +84,17 @@ SetSysname(ahost, auid, sysname)
 }
 
 
-static
-GetTokens(ahost, auid)
-     afs_int32 ahost;
-     afs_int32 auid;
+static int
+GetTokens(afs_int32 ahost, afs_int32 auid)
 {
     struct ViceIoctl iob;
     afs_int32 pheader[6];
     char tbuffer[1024];
-    register afs_int32 code;
+    afs_int32 code = 0;
     int index, newIndex;
     char *stp;			/* secret token ptr */
     struct ClearToken ct;
-    register char *tp;
+    char *tp;
     afs_int32 temp, gotit = 0;
     int maxLen;			/* biggest ticket we can copy */
     int tktLen;			/* server ticket length */
@@ -231,10 +218,8 @@ GetTokens(ahost, auid)
 }
 
 
-static
-NFSUnlog(ahost, auid)
-     afs_int32 ahost;
-     afs_int32 auid;
+static int
+NFSUnlog(afs_int32 ahost, afs_int32 auid)
 {
     afs_int32 code;
     afs_int32 pheader[6];
@@ -265,10 +250,8 @@ NFSUnlog(ahost, auid)
 }
 
 /* Copy the AFS service token into the kernel for a particular host and user */
-static
-NFSCopyToken(ahost, auid)
-     afs_int32 ahost;
-     afs_int32 auid;
+static int
+NFSCopyToken(afs_int32 ahost, afs_int32 auid)
 {
     struct ktc_principal client, server;
     struct ktc_token theTicket;
@@ -350,12 +333,12 @@ NFSCopyToken(ahost, auid)
 }
 
 static int
-cmdproc(register struct cmd_syndesc *as, void *arock)
+cmdproc(struct cmd_syndesc *as, void *arock)
 {
-    register struct hostent *the;
+    struct hostent *the;
     char *tp, *sysname = 0;
     afs_int32 uid, addr;
-    register afs_int32 code;
+    afs_int32 code;
 
     the = (struct hostent *)
 	hostutil_GetHostByName(tp = as->parms[0].items->data);
@@ -435,17 +418,16 @@ cmdproc(register struct cmd_syndesc *as, void *arock)
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char **argv)
 {
-    register struct cmd_syndesc *ts;
-    register afs_int32 code;
+    struct cmd_syndesc *ts;
+    afs_int32 code;
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
