@@ -9,16 +9,14 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/libadmin/adminutil/afs_utilAdmin.c,v 1.9.4.3 2009/03/16 13:26:48 shadow Exp $");
-
 #include <afs/stds.h>
+
+#include <roken.h>
+
+#include <rx/rx.h>
+#include <rx/rxstat.h>
+
 #include <afs/afs_Admin.h>
-#include <stdio.h>
-#include <string.h>
-#include <afs/afs_Admin.h>
-#include "afs_AdminInternal.h"
 #include <afs/pthread_glock.h>
 #include <afs/cellconfig.h>
 #include <afs/dirpath.h>
@@ -29,18 +27,13 @@ RCSID
 #include <afs/pterror.h>
 #include <afs/bnode.h>
 #include <afs/volser.h>
-#include <afs/afsint.h>
-#include <rx/rx.h>
-#include <rx/rxstat.h>
+#include <afs/afscbint.h>
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
+# include <afs/krb5_nt.h>
 #endif
+
+#include "afs_AdminInternal.h"
 #include "afs_utilAdmin.h"
 
 /*
@@ -81,6 +74,9 @@ init_once(void)
     initialize_AU_error_table();
     initialize_AV_error_table();
     initialize_VOLS_error_table();
+#ifdef AFS_KRB5_ERROR_ENV
+    initialize_krb5();
+#endif
     error_init_done = 1;
 }
 
@@ -105,6 +101,12 @@ util_AdminErrorCodeTranslate(afs_status_t errorCode, int langId,
 	pthread_once(&error_init_once, init_once);
     code = (afs_int32) errorCode;
     *errorTextP = afs_error_message(code);
+#ifdef AFS_KRB5_ERROR_ENV
+    if (strncmp(*errorTextP, "unknown", strlen("unknown")) == 0) {
+        const char *msg = fetch_krb5_error_message(NULL, code);
+        *errorTextP = msg ? msg : error_message(code);
+    }
+#endif
     rc = 1;
 
   fail_util_AdminErrorCodeTranslate:
@@ -1199,13 +1201,13 @@ UnmarshallRPCStats(afs_uint32 serverVersion, afs_uint32 ** ptrP,
     s->stats_v1.func_index = *(ptr++);
     hi = *(ptr++);
     lo = *(ptr++);
-    hset64(s->stats_v1.invocations, hi, lo);
+    s->stats_v1.invocations = ((afs_uint64) hi << 32) + lo;
     hi = *(ptr++);
     lo = *(ptr++);
-    hset64(s->stats_v1.bytes_sent, hi, lo);
+    s->stats_v1.bytes_sent = ((afs_uint64) hi << 32) + lo;
     hi = *(ptr++);
     lo = *(ptr++);
-    hset64(s->stats_v1.bytes_rcvd, hi, lo);
+    s->stats_v1.bytes_rcvd = ((afs_uint64) hi << 32) + lo;
     s->stats_v1.queue_time_sum.sec = *(ptr++);
     s->stats_v1.queue_time_sum.usec = *(ptr++);
     s->stats_v1.queue_time_sum_sqr.sec = *(ptr++);
@@ -2000,18 +2002,28 @@ ListCellsRPC(void *rpc_specific, int slot, int *last_item,
     afs_status_t tst = 0;
     cm_list_cell_get_p t = (cm_list_cell_get_p) rpc_specific;
     char *name;
+    serverList sl;
+    unsigned int n;
 
     /*
      * Get the next entry in the CellServDB.
      */
     name = t->cell[slot].cellname;
+    sl.serverList_len = 0;
+    sl.serverList_val = NULL;
+    memset(t->cell[slot].serverAddr, 0, sizeof(afs_int32)*UTIL_MAX_CELL_HOSTS);
     tst =
-	RXAFSCB_GetCellServDB(t->conn, t->index, &name,
-			      t->cell[slot].serverAddr);
+	RXAFSCB_GetCellServDB(t->conn, t->index, &name, &sl);
     if (tst) {
 	goto fail_ListCellsRPC;
     }
     strcpy(t->cell[slot].cellname, name);
+    if (sl.serverList_val) {
+        for (n=0; n<sl.serverList_len && n<UTIL_MAX_CELL_HOSTS; n++) {
+            t->cell[slot].serverAddr[n] = sl.serverList_val[n];
+        }
+        xdr_free((xdrproc_t) xdr_serverList, &sl);
+    }
 
     /*
      * See if we've processed all the entries
@@ -2292,7 +2304,7 @@ util_CMClientConfig(struct rx_connection *conn, afs_ClientConfig_p config,
 {
     int rc = 0;
     afs_status_t tst = 0;
-    afs_int32 allocbytes;
+    afs_uint32 allocbytes;
     struct cacheConfig tconfig;
 
     if (conn == NULL) {
