@@ -9,21 +9,12 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/libadmin/kas/afs_kasAdmin.c,v 1.13.4.1 2007/07/16 00:02:17 jaltman Exp $");
-
-#include <stdio.h>
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-
 #include <afs/stds.h>
+
+#include <roken.h>
+
+#include <rx/rx.h>
+#include <rx/rxstat.h>
 #include "afs_kasAdmin.h"
 #include "../adminutil/afs_AdminInternal.h"
 #include <afs/afs_AdminErrors.h>
@@ -1810,7 +1801,7 @@ kas_StringToKey(const char *cellName, const char *string,
     int rc = 0;
     afs_status_t tst = 0;
 
-    ka_StringToKey(string, cellName, (struct ktc_encryptionKey *)key);
+    ka_StringToKey((char *)string, (char *)cellName, (struct ktc_encryptionKey *)key);
     rc = 1;
 
     if (st != NULL) {
