@@ -5,36 +5,38 @@
  * may copy or modify Sun RPC without charge, but are not authorized
  * to license or distribute it to anyone else except as part of a product or
  * program developed by the user.
- * 
+ *
  * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
  * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
  * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
- * 
+ *
  * Sun RPC is provided with no support and without any obligation on the
  * part of Sun Microsystems, Inc. to assist in its use, correction,
  * modification or enhancement.
- * 
+ *
  * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
  * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
  * OR ANY PART THEREOF.
- * 
+ *
  * In no event will Sun Microsystems, Inc. be liable for any lost revenue
  * or profits or other special, indirect and consequential damages, even if
  * Sun has been advised of the possibility of such damages.
- * 
+ *
  * Sun Microsystems, Inc.
  * 2550 Garcia Avenue
  * Mountain View, California  94043
  */
 #include <afsconfig.h>
 #include <afs/param.h>
-#include "rx.h"
-
-RCSID
-    ("$Header: /cvs/openafs/src/rx/xdr_array.c,v 1.12.4.3 2009/01/28 22:33:28 shadow Exp $");
 
 #ifndef	NeXT
 
+#ifndef KERNEL
+# include <roken.h>
+#endif
+
+#include "rx.h"
+
 /*
  * xdr_array.c, Generic XDR routines impelmentation.
  *
@@ -44,21 +46,16 @@ RCSID
  * arrays.  See xdr.h for more info on the interface to xdr.
  */
 
-#ifdef	KERNEL
+#if defined(KERNEL) && !defined(UKERNEL)
+
 #include <sys/param.h>
 #ifdef AFS_LINUX20_ENV
 #include "h/string.h"
-#if 0
-#define bzero(A,C) memset((A), 0, (C))
-#endif
 #else
 #ifndef AFS_DARWIN90_ENV
 #include <sys/systm.h>
 #endif
 #endif /* AFS_LINUX20_ENV */
-#else
-#include <stdio.h>
-#include <string.h>
 #endif
 #include "xdr.h"
 
@@ -72,7 +69,7 @@ RCSID
  * elsize is the size (in bytes) of each element, and elproc is the
  * xdr procedure to call to handle each element of the array.
  */
-/* 
+/*
 	caddr_t *addrp;		* array pointer *
 	u_int *sizep;		* number of elements *
 	u_int maxsize;		* max numberof elements *
