@@ -5,23 +5,23 @@
  * may copy or modify Sun RPC without charge, but are not authorized
  * to license or distribute it to anyone else except as part of a product or
  * program developed by the user.
- * 
+ *
  * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
  * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
  * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
- * 
+ *
  * Sun RPC is provided with no support and without any obligation on the
  * part of Sun Microsystems, Inc. to assist in its use, correction,
  * modification or enhancement.
- * 
+ *
  * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
  * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
  * OR ANY PART THEREOF.
- * 
+ *
  * In no event will Sun Microsystems, Inc. be liable for any lost revenue
  * or profits or other special, indirect and consequential damages, even if
  * Sun has been advised of the possibility of such damages.
- * 
+ *
  * Sun Microsystems, Inc.
  * 2550 Garcia Avenue
  * Mountain View, California  94043
@@ -29,7 +29,12 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/rx/xdr_refernce.c,v 1.4 2001/08/08 00:03:57 shadow Exp $");
+#ifndef KERNEL
+# include <roken.h>
+#endif
+
+#include "rx.h"
+
 
 #ifndef	NeXT
 
@@ -44,12 +49,9 @@ RCSID("$Header: /cvs/openafs/src/rx/xdr_refernce.c,v 1.4 2001/08/08 00:03:57 sha
 
 #include "xdr.h"
 #include <stdio.h>
+#include <string.h>
 #define LASTUNSIGNED	((u_int)0-1)
 
-#ifndef osi_alloc
-char *osi_alloc();
-#endif
-
 /*
  * XDR an indirect pointer
  * xdr_reference is for recursively translating a structure that is
@@ -60,35 +62,36 @@ char *osi_alloc();
  * proc is the routine to handle the referenced structure.
  */
 bool_t
-xdr_reference(xdrs, pp, size, proc)
-	register XDR *xdrs;
-	caddr_t *pp;		/* the pointer to work on */
-	u_int size;		/* size of the object pointed to */
-	xdrproc_t proc;		/* xdr routine to handle the object */
+xdr_reference(XDR *xdrs,
+	      caddr_t *pp, /* the pointer to work on */
+	      u_int size, /* size of the object pointed to */
+	      xdrproc_t proc)  /* xdr routine to handle the object */
 {
-	register caddr_t loc = *pp;
-	register bool_t stat;
+    caddr_t loc = *pp;
+    bool_t stat;
 
-	if (loc == NULL)
-		switch (xdrs->x_op) {
-		case XDR_FREE:
-			return (TRUE);
+    if (loc == NULL)
+	switch (xdrs->x_op) {
+	case XDR_FREE:
+	    return (TRUE);
 
-		case XDR_DECODE:
-			*pp = loc = osi_alloc(size);
-			if (loc == NULL) {
-				return (FALSE);
-			}
-			memset(loc, 0, (int)size);
-			break;
+	case XDR_DECODE:
+	    *pp = loc = osi_alloc(size);
+	    if (loc == NULL) {
+		return (FALSE);
+	    }
+	    memset(loc, 0, (int)size);
+	    break;
+	case XDR_ENCODE:
+	    break;
 	}
 
-	stat = (*proc)(xdrs, loc, LASTUNSIGNED);
+    stat = (*proc) (xdrs, loc, LASTUNSIGNED);
 
-	if (xdrs->x_op == XDR_FREE) {
-		osi_free(loc, size);
-		*pp = NULL;
-	}
-	return (stat);
+    if (xdrs->x_op == XDR_FREE) {
+	osi_free(loc, size);
+	*pp = NULL;
+    }
+    return (stat);
 }
-#endif /* NeXT */
+#endif /* !NeXT */
