@@ -2,22 +2,22 @@
  * Copyright (c) 1995, 1996, 1997, 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- * 
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 
+ *
  * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -38,7 +38,7 @@
  *   require a specific license from the United States Government.
  *   It is the responsibility of any person or organization contemplating
  *   export to obtain such a license before exporting.
- * 
+ *
  * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
  * distribute this software and its documentation for any purpose and
  * without fee is hereby granted, provided that the above copyright
@@ -55,54 +55,29 @@
  */
 
 #include <afsconfig.h>
-#if defined(UKERNEL)
-#include "../afs/param.h"
-#else
 #include <afs/param.h>
-#endif
-
-RCSID
-    ("$Header: /cvs/openafs/src/rxkad/ticket5.c,v 1.10.2.1 2006/06/29 23:27:16 jaltman Exp $");
-
-#if defined(UKERNEL)
-#include "../afs/sysincludes.h"
-#include "../afs/afsincludes.h"
-#include "../afs/stds.h"
-#include "../rx/xdr.h"
-#include "../rx/rx.h"
-#include "../des/des.h"
-#include "../afs/lifetimes.h"
-#include "../afs/rxkad.h"
-#else /* defined(UKERNEL) */
 #include <afs/stds.h>
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
+
+#include <roken.h>
+
+#ifdef IGNORE_SOME_GCC_WARNINGS
+# pragma GCC diagnostic warning "-Wimplicit-function-declaration"
 #endif
+
 #include <rx/xdr.h>
 #include <rx/rx.h>
-#include <des.h>
+
+#include <hcrypto/md4.h>
+#include <hcrypto/md5.h>
+
 #include "lifetimes.h"
 #include "rxkad.h"
-#endif /* defined(UKERNEL) */
 
 #include "v5gen-rewrite.h"
-#include "asn1-common.h"
-#include "der.h"
 #include "v5gen.h"
+#include "der.h"
 #include "v5der.c"
 #include "v5gen.c"
-#include "md4.h"
-#include "md5.h"
 
 /*
  * Principal conversion Taken from src/lib/krb5/krb/conv_princ from MIT Kerberos.  If you
@@ -198,10 +173,10 @@ static int
 
 int
 tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
-		  int (*get_key) (char *, int, struct ktc_encryptionKey *),
+		  int (*get_key) (void *, int, struct ktc_encryptionKey *),
 		  char *get_key_rock, int serv_kvno, char *name, char *inst,
-		  char *cell, char *session_key, afs_int32 * host,
-		  afs_int32 * start, afs_int32 * end)
+		  char *cell, struct ktc_encryptionKey *session_key, afs_int32 * host,
+		  afs_uint32 * start, afs_uint32 * end, afs_int32 disableCheckdot)
 {
     char plain[MAXKRB5TICKETLEN];
     struct ktc_encryptionKey serv_key;
@@ -222,19 +197,19 @@ tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
 	return RXKADBADTICKET;	/* no ticket */
 
     if (serv_kvno == RXKAD_TKT_TYPE_KERBEROS_V5) {
-	code = decode_Ticket(ticket, ticket_len, &t5, &siz);
+	code = decode_Ticket((unsigned char *)ticket, ticket_len, &t5, &siz);
 	if (code != 0)
 	    goto cleanup;
 
 	if (t5.tkt_vno != 5)
 	    goto bad_ticket;
     } else {
-	code = decode_EncryptedData(ticket, ticket_len, &t5.enc_part, &siz);
+	code = decode_EncryptedData((unsigned char *)ticket, ticket_len, &t5.enc_part, &siz);
 	if (code != 0)
 	    goto cleanup;
     }
 
-    /* If kvno is null, it's probably not included because it was kvno==0 
+    /* If kvno is null, it's probably not included because it was kvno==0
      * in the ticket */
     if (t5.enc_part.kvno == NULL) {
 	v5_serv_kvno = 0;
@@ -270,7 +245,7 @@ tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
 	goto bad_ticket;
 
     /* Decode ticket */
-    code = decode_EncTicketPart(plain, plainsiz, &decr_part, &siz);
+    code = decode_EncTicketPart((unsigned char *)plain, plainsiz, &decr_part, &siz);
     if (code != 0)
 	goto bad_ticket;
 
@@ -318,13 +293,15 @@ tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
 	goto bad_ticket;
     }
 
-    /* 
-     * If the first part of the name_string contains a dot, punt since
-     * then we can't see the diffrence between the kerberos 5
-     * principals foo.root and foo/root later in the fileserver.
-     */
-    if (strchr(decr_part.cname.name_string.val[0], '.') != NULL)
-	goto bad_ticket;
+    if (!disableCheckdot) {
+        /*
+         * If the first part of the name_string contains a dot, punt since
+         * then we can't see the diffrence between the kerberos 5
+         * principals foo.root and foo/root later in the fileserver.
+         */
+        if (strchr(decr_part.cname.name_string.val[0], '.') != NULL)
+	    goto bad_ticket;
+    }
 
     /* Verify that decr_part.key is of right type */
     switch (decr_part.key.keytype) {
@@ -444,15 +421,15 @@ krb5_des_decrypt(struct ktc_encryptionKey *key, int etype, void *in,
 {
     int (*cksum_func) (void *, size_t, void *, size_t,
 		       struct ktc_encryptionKey *);
-    des_cblock ivec;
-    des_key_schedule s;
+    DES_cblock ivec;
+    DES_key_schedule s;
     char cksum[24];
     size_t cksumsz;
     int ret = 1;		/* failure */
 
     cksum_func = NULL;
 
-    des_key_sched(key, &s);
+    DES_key_sched(ktc_to_cblock(key), &s);
 
 #define CONFOUNDERSZ 8
 
@@ -476,7 +453,7 @@ krb5_des_decrypt(struct ktc_encryptionKey *key, int etype, void *in,
 	abort();
     }
 
-    des_cbc_encrypt(in, out, insz, s, &ivec, 0);
+    DES_cbc_encrypt(in, out, insz, &s, &ivec, 0);
 
     memcpy(cksum, (char *)out + CONFOUNDERSZ, cksumsz);
     memset((char *)out + CONFOUNDERSZ, 0, cksumsz);
