@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -12,8 +12,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/util/winsock_nt.c,v 1.6 2005/11/05 06:48:27 jaltman Exp $");
+#include <roken.h>
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -22,21 +21,16 @@ RCSID
 
 /* afs_wsInit - LWP version
  * Initialize the winsock 2 environment.
- * 
+ *
  * Returns 0 on success, -1 on error.
  */
 int
 afs_winsockInit(void)
 {
-    static int once = 1;
-
-    if (once) {
 	int code;
 	WSADATA data;
 	WORD sockVersion;
 
-	once = 0;
-
 	sockVersion = 2;
 	code = WSAStartup(sockVersion, &data);
 	if (code)
@@ -44,23 +38,19 @@ afs_winsockInit(void)
 
 	if (data.wVersion != 2)
 	    return -1;
-    }
     return 0;
 }
 
-/* This function will begin to fail in the year 2038 */
+void
+afs_winsockCleanup(void)
+{
+    WSACleanup();
+}
+
+/* exported from libafsauthent.dll */
 int
 afs_gettimeofday(struct timeval *tv, struct timezone *tz)
 {
-    struct _timeb myTime;
-
-    _ftime(&myTime);
-    tv->tv_sec = myTime.time;
-    tv->tv_usec = myTime.millitm * 1000;
-    if (tz) {
-	tz->tz_minuteswest = myTime.timezone;
-	tz->tz_dsttime = myTime.dstflag;
-    }
-    return 0;
+    return rk_gettimeofday(tv, tz);
 }
 #endif
